#!/bin/sh

if [ ! -f /usr/share/common-lisp/source/defsystem/defsystem.lisp ] ; then
  cat <<EOF
$0: cannot find the defsystem source.
This can not happen, because defsystem is part of the common-lisp-controller package. 
EOF
  exit 0
fi

case $1 in
    rebuild)
	    echo $0 rebuilding...
	    shift
	    while [ ! -z "$1" ] ; do
		echo rebuilding $1
		sbcl --noinform --sysinit /etc/sbclrc --userinit /dev/null \
		     --noprogrammer \
		     --eval "(funcall (intern \"OOS\" (find-package \"MAKE\")) :$1 :compile :verbose nil :compile-during-load t)" \
		     --eval "(quit)"
		shift
 	    done 
	    ;;
     remove)
	    echo $0 removing packages...
	    shift
	    while [ ! -z "$1" ] ; do
		rm -rf "/usr/lib/common-lisp/sbcl/$1"
		shift
 	    done
	    ;;
    install-defsystem)
	    echo $0 loading and dumping defsystem.
	    ( cd /usr/lib/sbcl
	      if [ ! -f sbcl-clean.core ]; then
	        mv sbcl.core sbcl-clean.core
	      fi
	      sbcl --core /usr/lib/sbcl/sbcl-clean.core \
		   --noinform --sysinit /dev/null --userinit /dev/null \
		   --eval "(load \"/usr/share/common-lisp/source/common-lisp-controller/common-lisp-controller.lisp\")" \
		   --eval "(funcall (intern \"INIT-COMMON-LISP-CONTROLLER\" \"COMMON-LISP-CONTROLLER\") \"/usr/lib/common-lisp/sbcl/\")" \
		   --eval "(ensure-directories-exist \"/usr/lib/common-lisp/sbcl/defsystem/\")" \
		   --eval "(load (compile-file \"cl-library:defsystem;defsystem.lisp\"))" \
		   --eval "(push \"cl-systems:\" (symbol-value (intern \"*CENTRAL-REGISTRY*\" \"MAKE\")))" \
		   --eval "(sb-ext:save-lisp-and-die \"sbcl.core\")" )
	    ;;
    remove-defsystem)
	    echo $0 removing defsystem-enabled image
	    if [ -f /usr/lib/sbcl/sbcl-clean.core ]; then
	      mv /usr/lib/sbcl/sbcl-clean.core /usr/lib/sbcl/sbcl.core
	    fi
	    ;;
    *)
	    echo $0 unkown command $1
	    echo known commands: rebuild, remove, install-defsystem and remove-defsystem
	    exit 1
	    ;;
esac

exit 0
